
from setuptools import setup, find_packages
setup(name='atcbaselib',
      version='0.0.3',
      description='Log check and auto rdm bug',
      author='zw',
      author_email='zwwwww@163.com',
      requires= ['numpy','matplotlib', 're', 'os', 'datetime', 'csv', 'fuzzywuzzy'], # 定义依赖哪些模块
      packages=find_packages(),  # 系统自动从当前目录开始找包
      # 如果有的文件不用打包，则只能指定需要打包的文件
      #packages=['代码1','代码2','__init__']  #指定目录中需要打包的py文件，注意不要.py后缀
      license="apache 3.0",
      data_files=['atcbaselib/logconfig.ini']
      )